@@ -1,4 +1,4 @@
-/* $Id: ekg.c,v 1.348.2.4 2007-04-28 20:02:23 porridge Exp $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -149,6 +149,10 @@ static struct {
 	EKG_HANDLER(GG_SESSION_DCC_SEND, handle_dcc, gg_dcc_free)
 	EKG_HANDLER(GG_SESSION_DCC_GET, handle_dcc, gg_dcc_free)
 	EKG_HANDLER(GG_SESSION_DCC_VOICE, handle_dcc, gg_dcc_free)
+	EKG_HANDLER(GG_SESSION_DCC7_SOCKET, handle_dcc7, gg_dcc7_free)
+	EKG_HANDLER(GG_SESSION_DCC7_SEND, handle_dcc7, gg_dcc7_free)
+	EKG_HANDLER(GG_SESSION_DCC7_GET, handle_dcc7, gg_dcc7_free)
+	EKG_HANDLER(GG_SESSION_DCC7_VOICE, handle_dcc7, gg_dcc7_free)
 	EKG_HANDLER(GG_SESSION_REGISTER, handle_pubdir, gg_register_free)
 	EKG_HANDLER(GG_SESSION_UNREGISTER, handle_pubdir, gg_pubdir_free)
 	EKG_HANDLER(GG_SESSION_PASSWD, handle_pubdir, gg_change_passwd_free)
@@ -404,6 +408,7 @@ void ekg_wait_for_key()
 			struct gg_common *c = l->data;
 			struct gg_http *h = l->data;
 			struct gg_dcc *d = l->data;
+			struct gg_dcc7 *d7 = l->data;
 			static time_t last_check = 0;
 
 			if (!c || c->timeout == -1 || time(NULL) == last_check)
@@ -498,6 +503,32 @@ void ekg_wait_for_key()
 					gg_free_dcc(d);
 					break;
 				}
+
+				case GG_SESSION_DCC7_GET:
+				case GG_SESSION_DCC7_SEND:
+				{
+					struct in_addr addr;
+					unsigned short port = d7->remote_port;
+					char *tmp;
+			
+					addr.s_addr = d7->remote_addr;
+
+					if (d7->peer_uin) {
+						struct userlist *u = userlist_find(d7->peer_uin, NULL);
+						if (!addr.s_addr && u) {
+							addr.s_addr = u->ip.s_addr;
+							port = u->port;
+						}
+						tmp = saprintf("%s (%s:%d)", format_user(d7->peer_uin), inet_ntoa(addr), port);
+					} else 
+						tmp = saprintf("%s:%d", inet_ntoa(addr), port);
+					print("dcc_timeout", tmp);
+					xfree(tmp);
+					remove_transfer(d7);
+					list_remove(&watches, d7, 0);
+					gg_dcc7_free(d7);
+					break;
+				}
 			}
 
 			break;
@@ -586,7 +617,7 @@ void ekg_wait_for_key()
 			
 					if (GG_S_I(u->status)) {
 						int status = (GG_S_D(u->status)) ? GG_STATUS_NOT_AVAIL_DESCR : GG_STATUS_NOT_AVAIL;
-						iso_to_cp(u->descr);
+						iso_to_cp((unsigned char *) u->descr);
 						handle_common(u->uin, status, u->descr, time(NULL), u->ip.s_addr, u->port, u->protocol, u->image_size);
 
 						if (tmp) {
@@ -637,10 +668,17 @@ void ekg_wait_for_key()
 				FD_SET(w->fd, &wd);
 		}
 
+#ifdef WITH_UI_GTK
+		if (ui_init == ui_gtk_init) {
+			tv.tv_sec = 0;
+			tv.tv_usec = 20000;
+		} else 
+#endif
+		{
 		/* domylny timeout to 1s */
-		
-		tv.tv_sec = 1;
-		tv.tv_usec = 0;
+			tv.tv_sec = 1;
+			tv.tv_usec = 0;
+		}
 
 		/* ale jeli ktry timer ma wystpi wczeniej ni za sekund
 		 * to skrmy odpowiednio czas oczekiwania */
@@ -745,7 +783,10 @@ void ekg_wait_for_key()
 		if (!ret) {
 			if (batch_mode && !batch_line)
 				break;
-
+#ifdef WITH_UI_GTK
+			if (ui_init == ui_gtk_init)
+				break;
+#endif
 			continue;
 		}
 
@@ -861,7 +902,6 @@ void ekg_wait_for_key()
 				}
 			}
 		}
-
 	}
 	
 	return;
@@ -893,7 +933,7 @@ static void ioctld_kill()
                 kill(ioctld_pid, SIGINT);
 }
 
-#ifdef HAVE_EXECINFO_H
+#ifdef HAVE_BACKTRACE
 static void dump_stack(void)
 {
 	char name[32];
@@ -935,7 +975,7 @@ static void handle_sigsegv(int sig)
 	userlist_write_crash();
 	debug_write_crash();
 
-#ifdef HAVE_EXECINFO_H
+#ifdef HAVE_BACKTRACE
 	dump_stack();
 #endif
 
@@ -987,6 +1027,7 @@ static void debug_handler(int level, const char *format, va_list ap)
 {
 	static string_t line = NULL;
 	char *tmp;
+	char *theme_format;
 
 	tmp = gg_vsaprintf(format, ap);
 
@@ -1010,11 +1051,40 @@ static void debug_handler(int level, const char *format, va_list ap)
 	if (!tmp)
 		return;
 
+	switch(level) {
+		/* nieuzywane? */
+		/*
+		case GG_DEBUG_NET 	 1:
+			theme_format = "debug";
+			break;
+		*/
+
+		case /* GG_DEBUG_TRAFFIC */ 	 2:
+			theme_format = "iodebug";
+			break;
+
+		case /* GG_DEBUG_DUMP */	 4:
+			theme_format = "iodebug";
+		 	break;
+
+		case /* GG_DEBUG_FUNCTION */	 8:
+			theme_format = "fdebug";
+		 	break;
+
+		case /* GG_DEBUG_MISC */	16:
+			theme_format = "debug";
+			break;
+
+		default:
+			theme_format = "debug";
+			break;
+	}
+
 	tmp[strlen(tmp) - 1] = 0;
 
 	buffer_add(BUFFER_DEBUG, NULL, tmp, DEBUG_MAX_LINES);
 	if (ui_print)
-		print_window("__debug", 0, "debug", tmp);
+		print_window("__debug", 0, theme_format, tmp);
 	xfree(tmp);
 }
 #endif
@@ -1042,6 +1112,10 @@ static int ekg_ui_set(const char *name)
 #ifdef WITH_UI_NCURSES
 	else if (!strcasecmp(name, "ncurses"))
 		ui_init = ui_ncurses_init;
+#endif
+#ifdef WITH_UI_GTK
+	else if (!strcasecmp(name, "gtk"))
+		ui_init = ui_gtk_init;
 #endif
 	else
 		return -1;
@@ -1067,6 +1141,8 @@ int main(int argc, char **argv)
 	struct option ekg_options[] = {
 		{ "back", optional_argument, 0, 'b' },
 		{ "away", optional_argument, 0, 'a' },
+		{ "ffc", optional_argument, 0, 'F' },
+		{ "dnd", optional_argument, 0, 'd' },
 		{ "invisible", optional_argument, 0, 'i' },
 		{ "private", no_argument, 0, 'p' },
 		{ "no-auto", no_argument, 0, 'n' },
@@ -1079,6 +1155,7 @@ int main(int argc, char **argv)
 		{ "user", required_argument, 0, 'u' },
 		{ "version", no_argument, 0, 'v' },
 		{ "no-global-config", no_argument, 0, 'N' },
+		{ "no-autorun", no_argument, 0, 'A' },
 		{ 0, 0, 0, 0 }
 	};
 
@@ -1095,6 +1172,8 @@ int main(int argc, char **argv)
 	ui_init = ui_readline_init;
 #elif defined(WITH_UI_NCURSES)
 	ui_init = ui_ncurses_init;
+#elif defined(WITH_UI_GTK)
+	ui_init = ui_gtk_init;
 #else
 	ui_init = ui_batch_init;
 #endif
@@ -1142,7 +1221,7 @@ int main(int argc, char **argv)
 	sa.sa_handler = handle_sigusr2;
 	sigaction(SIGUSR2, &sa, NULL);
 
-	while ((c = getopt_long(argc, argv, "b::a::i::pdnc:f:hI:ot:u:vN", ekg_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "b::a::i::F::d::pnc:f:hI:ot:u:vNA", ekg_options, NULL)) != -1) {
 		switch (c) {
 			case 'b':
 				if (!optarg && argv[optind] && argv[optind][0] != '-')
@@ -1160,6 +1239,22 @@ int main(int argc, char **argv)
 				xfree(new_reason);
 				new_reason = xstrdup(optarg);
 			        break;
+			case 'F':
+				if (!optarg && argv[optind] && argv[optind][0] != '-')
+					optarg = argv[optind++];
+
+				new_status = (optarg) ? GG_STATUS_FFC_DESCR : GG_STATUS_FFC;
+				xfree(new_reason);
+				new_reason = xstrdup(optarg);
+			        break;
+			case 'd':
+				if (!optarg && argv[optind] && argv[optind][0] != '-')
+					optarg = argv[optind++];
+
+				new_status = (optarg) ? GG_STATUS_DND_DESCR : GG_STATUS_DND;
+				xfree(new_reason);
+				new_reason = xstrdup(optarg);
+			        break;
 			case 'i':
 				if (!optarg && argv[optind] && argv[optind][0] != '-')
 					optarg = argv[optind++];
@@ -1181,6 +1276,7 @@ int main(int argc, char **argv)
 				printf(""
 "uycie: %s [OPCJE] [KOMENDY]\n"
 "  -N, --no-global-config     ignoruje globalny plik konfiguracyjny\n"
+"  -A, --no-autorun           nie uruchamia aliasu ,,autorun'' przy starcie\n"
 "  -u, --user=NAZWA           korzysta z profilu uytkownika o podanej nazwie\n"
 "  -t, --theme=PLIK           aduje opis wygldu z podanego pliku\n"
 "  -c, --control-pipe=PLIK    potok nazwany sterowania\n"
@@ -1188,6 +1284,8 @@ int main(int argc, char **argv)
 "  -n, --no-auto              nie czy si automatycznie z serwerem\n"
 "  -a, --away[=OPIS]          domylnie zmienia stan na ,,zajty''\n"
 "  -b, --back[=OPIS]          domylnie zmienia stan na ,,dostpny''\n"
+"  -F, --ffc[=OPIS]           domylnie zmienia stan na ,,poGGadaj ze mn''\n"
+"  -d, --dnd[=OPIS]           domylnie zmienia stan na ,,nie przeszkadza''\n"
 "  -i, --invisible[=OPIS]     domylnie zmienia stan na ,,niewidoczny''\n"
 "  -p, --private              domylnie ustawia tryb ,,tylko dla znajomych''\n"
 "  -v, --version              wywietla wersje programu i wychodzi\n"
@@ -1202,6 +1300,9 @@ int main(int argc, char **argv)
 #ifdef WITH_UI_NCURSES
 ", ncurses"
 #endif
+#ifdef WITH_UI_GTK
+", gtk"
+#endif
 ")\n"
 "\n", argv[0]);
 				return 0;
@@ -1221,10 +1322,13 @@ int main(int argc, char **argv)
 			case 'v':
 			    	printf("ekg-%s\nlibgadu-%s (headers %s, protocol 0x%.2x, client \"%s\")\ncompile time: %s\n", VERSION, gg_libgadu_version(), GG_LIBGADU_VERSION, GG_DEFAULT_PROTOCOL_VERSION, GG_DEFAULT_CLIENT_VERSION, compile_time());
 				return 0;
+			case 'A':
+				no_autorun = 1;
+				break;
 #ifdef WITH_IOCTLD
 			case 'I':
 				ioctld_path = optarg;
-			break;
+				break;
 #endif
 			case 'f':
 				ui_set = 1;
@@ -1304,6 +1408,16 @@ int main(int argc, char **argv)
 	}
 #endif
 
+#ifdef WITH_UI_GTK
+	if (ui_init == ui_gtk_init) {
+#ifndef GG_DEBUG_DISABLE
+		if (!gg_debug_file)
+			gg_debug_handler = debug_handler;
+#endif
+		gg_debug_level = 255;
+	}
+#endif
+
         ekg_pid = getpid();
 	mesg_startup = mesg_set(MESG_CHECK);
 
@@ -1342,7 +1456,7 @@ int main(int argc, char **argv)
 	"Do pliku %s/debug.%d zapisz ostatanie komunikaty\r\n"
 	"z okna debugowania.\r\n"
 	"\r\n"
-#ifdef HAVE_EXECINFO_H
+#ifdef HAVE_BACKTRACE
 	"Jeli zostanie utworzony plik %s/stack.%d, to uruchom\r\n"
 	"polecenie:\r\n"
 	"\r\n"
@@ -1533,11 +1647,25 @@ int main(int argc, char **argv)
 #endif
 
 	changed_dcc("dcc");
+	changed_dcc("dcc_ip");
 
 #ifdef WITH_PYTHON
 	python_autorun();
 #endif
 
+	if (!no_autorun) {
+		list_t l;
+
+		for (l = aliases; l; l = l->next) {
+			struct alias *a = l->data;
+			
+			if (!strcasecmp("autorun", a->name)) {
+				command_exec(NULL, "/autorun", 0);
+				break;
+			}
+		}
+	}
+
 	if (config_uin && config_password && auto_connect) {
 		print("connecting");
 		connecting = 1;
