@@ -1,4 +1,4 @@
-/* $Id: python.c,v 1.35.2.1 2006-08-24 18:57:56 porridge Exp $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2002-2003 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -486,7 +486,8 @@ int python_load(const char *name, int quiet)
 				}
 
 				Py_XDECREF(result);
-			}
+			} else
+				PyErr_Print();
 		}
 
 		Py_XDECREF(init);
