@@ -1,4 +1,4 @@
-/* $Id: stuff.c,v 1.448.2.4 2007-04-28 20:02:24 porridge Exp $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -96,6 +96,7 @@ list_t spiedlist = NULL;
 
 int command_processing = 0;
 int in_autoexec = 0;
+int no_autorun = 0;
 int in_auto_away = 0;
 int config_auto_reconnect = 10;
 int reconnect_timer = 0;
@@ -113,6 +114,10 @@ int config_beep_notify = 1;
 int config_beep_mail = 1;
 int config_display_pl_chars = 1;
 int config_events_delay = 3;
+int config_files_mode_config_int = 600;	/* nie 0600, bo jest zamieniane z dec na oct */
+int config_files_mode_config = 0600;		/* wartosc oktalna files_mode_config */
+int config_files_mode_received_int = 600;
+int config_files_mode_received = 0600;
 char *config_sound_msg_file = NULL;
 char *config_sound_chat_file = NULL;
 char *config_sound_sysmsg_file = NULL;
@@ -153,6 +158,8 @@ char *home_dir = NULL;
 char *config_quit_reason = NULL;
 char *config_away_reason = NULL;
 char *config_back_reason = NULL;
+char *config_ffc_reason = NULL;
+char *config_dnd_reason = NULL;
 int config_random_reason = 0;
 #ifdef HAVE_REGEX_H
 int config_regex_flags = 0;
@@ -189,6 +196,9 @@ char *config_email = NULL;
 int config_time_deviation = 300;
 int config_msg_as_chat = 0;
 int config_mesg = MESG_DEFAULT;
+#ifdef WITH_UI_NCURSES
+int config_mouse = 0;
+#endif
 char *config_nick = NULL;
 int config_display_welcome = 1;
 int config_auto_back = 0;
@@ -410,6 +420,8 @@ void alias_free()
 	aliases = NULL;
 }
 
+static const char base64_charset[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
 /*
  * base64_encode()
  *
@@ -417,15 +429,52 @@ void alias_free()
  */
 char *base64_encode(const char *buf)
 {
-	char *tmp = gg_base64_encode(buf);
+	char *out, *res;
+	unsigned int i = 0, j = 0, k = 0, len;
 
 	if (!buf)
 		return xstrdup("");
 
-	if (!tmp)
-		ekg_oom_handler();
+	len = strlen(buf);
 
-	return tmp;
+	res = out = xmalloc((len / 3 + 1) * 4 + 2);
+
+	while (j <= len) {
+		switch (i % 4) {
+			case 0:
+				k = (buf[j] & 252) >> 2;
+				break;
+			case 1:
+				if (j < len)
+					k = ((buf[j] & 3) << 4) | ((buf[j + 1] & 240) >> 4);
+				else
+					k = (buf[j] & 3) << 4;
+
+				j++;
+				break;
+			case 2:
+				if (j < len)
+					k = ((buf[j] & 15) << 2) | ((buf[j + 1] & 192) >> 6);
+				else
+					k = (buf[j] & 15) << 2;
+
+				j++;
+				break;
+			case 3:
+				k = buf[j++] & 63;
+				break;
+		}
+		*out++ = base64_charset[k];
+		i++;
+	}
+
+	if (i % 4)
+		for (j = 0; j < 4 - (i % 4); j++, out++)
+			*out = '=';
+
+	*out = 0;
+
+	return res;
 }
 
 /*
@@ -435,15 +484,48 @@ char *base64_encode(const char *buf)
  */
 char *base64_decode(const char *buf)
 {
-	char *tmp = gg_base64_decode(buf);
+	char *res, *save, val;
+	const char *foo;
+	const char *end;
+	unsigned int index = 0;
 
 	if (!buf)
 		return xstrdup("");
 
-	if (!tmp)
-		ekg_oom_handler();
+	save = res = xcalloc(1, (strlen(buf) / 4 + 1) * 3 + 2);
+	end = buf + strlen(buf);
 
-	return tmp;
+	while (*buf && buf < end) {
+		if (*buf == '\r' || *buf == '\n') {
+			buf++;
+			continue;
+		}
+		if (!(foo = strchr(base64_charset, *buf)))
+			foo = base64_charset;
+		val = (int)(foo - base64_charset);
+		buf++;
+		switch (index) {
+			case 0:
+				*res |= val << 2;
+				break;
+			case 1:
+				*res++ |= val >> 4;
+				*res |= val << 4;
+				break;
+			case 2:
+				*res++ |= val >> 2;
+				*res |= val << 6;
+				break;
+			case 3:
+				*res++ |= val;
+				break;
+		}
+		index++;
+		index %= 4;
+	}
+	*res = 0;
+
+	return save;
 }
 
 /*
@@ -905,6 +987,10 @@ void changed_xxx_reason(const char *var)
 		tmp = config_away_reason;
 	if (!strcmp(var, "back_reason"))
 		tmp = config_back_reason;
+	if (!strcmp(var, "dnd_reason"))
+		tmp = config_dnd_reason;
+	if (!strcmp(var, "ffc_reason"))
+		tmp = config_ffc_reason;
 	if (!strcmp(var, "quit_reason"))
 		tmp = config_quit_reason;
 
@@ -915,6 +1001,22 @@ void changed_xxx_reason(const char *var)
 		print("descr_too_long", itoa(strlen(tmp) - GG_STATUS_DESCR_MAXSIZE));
 }
 
+/*
+ * changed_files_mode()
+ *
+ * funkcja wywowana przy zmianie config_files_mode_*_int
+ */
+void changed_files_mode(const char *var)
+{
+	char mode[16];
+
+	snprintf(mode, sizeof(mode), "%d", config_files_mode_config_int);
+	config_files_mode_config = strtol(mode, NULL, 8);
+
+	snprintf(mode, sizeof(mode), "%d", config_files_mode_received_int);
+	config_files_mode_received = strtol(mode, NULL, 8);
+}
+
 /*
  * conference_add()
  *
@@ -1367,6 +1469,7 @@ void ekg_connect()
 #endif
 	p.protocol_version = config_protocol;
 	p.last_sysmsg = config_last_sysmsg;
+	p.protocol_features = GG_FEATURE_STATUS80 | GG_FEATURE_DND_FFC;	/* bez GG_FEATURE_MSG80 */
 
 	if (config_server) {
 		char *server, *sserver, *tmp, **servers = array_make(config_server, ",; ", 0, 1, 0);
@@ -1426,9 +1529,9 @@ skip_server:
 	}
 
 	if (!config_password_cp1250)
-		iso_to_cp(p.password);
+		iso_to_cp((unsigned char *) p.password);
 	if (p.status_descr)
-		iso_to_cp(p.status_descr);
+		iso_to_cp((unsigned char *) p.status_descr);
 
 	if (!(sess = gg_login(&p))) {
 		print("conn_failed", format_find((errno == ENOMEM) ? "conn_failed_memory" : "conn_failed_connecting"));
@@ -1437,9 +1540,9 @@ skip_server:
 		list_add(&watches, sess, 0);
 
 	if (!config_password_cp1250)
-		cp_to_iso(p.password);
+		cp_to_iso((unsigned char *) p.password);
 	if (p.status_descr)
-		cp_to_iso(p.status_descr);
+		cp_to_iso((unsigned char *) p.status_descr);
 }
 
 /*
@@ -1472,7 +1575,7 @@ void ekg_logoff(struct gg_session *sess, const char *reason)
 
 	if (reason) {
 		char *tmp = xstrdup(reason);
-		iso_to_cp(tmp);
+		iso_to_cp((unsigned char *) tmp);
 		gg_change_status_descr(sess, GG_STATUS_NOT_AVAIL_DESCR, tmp);
 		xfree(tmp);
 	} else
@@ -1913,7 +2016,7 @@ int msg_encrypt(uin_t uin, unsigned char **msg)
 {
 #ifdef HAVE_OPENSSL
 	if (config_encryption == 1 || config_encryption == 3) {
-		unsigned char *res = sim_message_encrypt(*msg, uin);
+		unsigned char *res = (unsigned char *) sim_message_encrypt(*msg, uin);
 
 		if (res) {
 			int ret = 1;
@@ -1921,7 +2024,7 @@ int msg_encrypt(uin_t uin, unsigned char **msg)
 			xfree(*msg);
 			*msg = res;
 			
-			if (strlen(*msg) > 1989) {
+			if (strlen((char *) *msg) > 1989) {
 				(*msg)[1989] = 0;
 				ret = 2;
 			}
@@ -2010,7 +2113,15 @@ void cp_to_iso(unsigned char *buf)
 			0x9F, 0xBC,	/*  */
 			0xA5, 0xA1,	/*  */
 			0x8C, 0xA6,	/*  */
-			0x8F, 0xAC	/*  */
+			0x8F, 0xAC,	/*  */
+			0x9A, 0xB9,	/*  */
+			0x8A, 0xA9,	/*  */
+			0x9E, 0xBE,	/*  */
+			0x8E, 0xAE,	/*  */
+			0x9D, 0xBB,	/*  */
+			0x8D, 0xAB,	/*  */
+			0xBE, 0xB5,	/*  */
+			0xBC, 0xA5,	/*  */
 		};
 		size_t i;
 
@@ -2033,6 +2144,168 @@ void cp_to_iso(unsigned char *buf)
 	charmap_convert(charmap, buf);
 }
 
+static const unsigned short table_iso_8859_2[] = {
+	'?', '?', '?', '?', '?', '?', '?', '?',					/* 0x80 -      */
+	'?', '?', '?', '?', '?', '?', '?', '?',					/*      - 0x8F */
+	'?', '?', '?', '?', '?', '?', '?', '?',					/* 0x90 -      */
+	'?', '?', '?', '?', '?', '?', '?', '?',					/*      - 0x9F */
+	0x00a0, 0x0104, 0x02d8, 0x0141, 0x00a4, 0x013d, 0x015a, 0x00a7,		/* 0xA0 -      */
+	0x00a8, 0x0160, 0x015e, 0x0164, 0x0179, 0x00ad, 0x017d, 0x017b,		/*      - 0xAF */
+	0x00b0, 0x0105, 0x02db, 0x0142, 0x00b4, 0x013e, 0x015b, 0x02c7,		/* 0xB0 -      */
+	0x00b8, 0x0161, 0x015f, 0x0165, 0x017a, 0x02dd, 0x017e, 0x017c,		/*      - 0xBF */
+	0x0154, 0x00c1, 0x00c2, 0x0102, 0x00c4, 0x0139, 0x0106, 0x00c7,		/* 0xC0 -      */
+	0x010c, 0x00c9, 0x0118, 0x00cb, 0x011a, 0x00cd, 0x00ce, 0x010e,		/*      - 0xCF */
+	0x0110, 0x0143, 0x0147, 0x00d3, 0x00d4, 0x0150, 0x00d6, 0x00d7,		/* 0xD0 -      */
+	0x0158, 0x016e, 0x00da, 0x0170, 0x00dc, 0x00dd, 0x0162, 0x00df,		/*      - 0xDF */
+	0x0155, 0x00e1, 0x00e2, 0x0103, 0x00e4, 0x013a, 0x0107, 0x00e7,		/* 0xE0 -      */
+	0x010d, 0x00e9, 0x0119, 0x00eb, 0x011b, 0x00ed, 0x00ee, 0x010f,		/*      - 0xEF */
+	0x0111, 0x0144, 0x0148, 0x00f3, 0x00f4, 0x0151, 0x00f6, 0x00f7,		/* 0xF0 -      */
+	0x0159, 0x016f, 0x00fa, 0x0171, 0x00fc, 0x00fd, 0x0163, 0x02d9		/*      - 0xFF */
+};
+
+/* utf8_to_iso() && iso_to_utf8() based on ekg2's gg_locale_to_cp() && gg_cp_to_locale() */
+
+/* some code based on libiconv utf8_mbtowc() && utf8_wctomb() from utf8.h under LGPL-2.1 */
+
+static int utf8_helper(unsigned char *s, int n, unsigned short *ch)
+{
+	unsigned char c = s[0];
+
+	if (c < 0x80) {
+		*ch = c;
+		return 1;
+	}
+
+	if (c < 0xc2) 
+		goto invalid;
+
+	if (c < 0xe0) {
+		if (n < 2)
+			goto invalid;
+		if (!((s[1] ^ 0x80) < 0x40))
+			goto invalid;
+		*ch = ((unsigned short) (c & 0x1f) << 6) | (unsigned short) (s[1] ^ 0x80);
+		return 2;
+	} 
+	
+	if (c < 0xf0) {
+		if (n < 3)
+			goto invalid;
+		if (!((s[1] ^ 0x80) < 0x40 && (s[2] ^ 0x80) < 0x40 && (c >= 0xe1 || s[1] >= 0xa0)))
+			goto invalid;
+		*ch = ((unsigned short) (c & 0x0f) << 12) | ((unsigned short) (s[1] ^ 0x80) << 6) | (unsigned short) (s[2] ^ 0x80);
+		return 3;
+	}
+
+invalid:
+	*ch = '?';
+	return 1;
+}
+
+/*
+ * utf8_to_iso()
+ *
+ * konwertuje utf8 na iso, zwalnia podany bufor i zwraca nowy.
+ */
+
+char *utf8_to_iso(char *b)
+{
+	unsigned char *buf = (unsigned char *) b;
+
+	char *newbuf;
+	int newlen = 0;
+	int len;
+	int i, j;
+
+	if (!buf)
+		return NULL;
+
+	len = strlen(b);
+
+	for (i = 0; i < len; newlen++) {
+		unsigned short discard;
+
+		i += utf8_helper(&buf[i], len - i, &discard);
+	}
+
+	newbuf = xmalloc(newlen+1);
+
+	for (i = 0, j = 0; buf[i]; j++) {
+		unsigned short znak;
+		int k;
+
+		i += utf8_helper(&buf[i], len - i, &znak);
+
+		if (znak < 0x80) {
+			newbuf[j] = znak;
+			continue;
+		}
+
+		newbuf[j] = '?';
+
+		for (k = 0; k < (sizeof(table_iso_8859_2)/sizeof(table_iso_8859_2[0])); k++) {
+			if (table_iso_8859_2[k] == znak) {
+				newbuf[j] = (0x80 | k);
+				break;
+			}
+		}
+	}
+	newbuf[j] = '\0';
+
+	xfree(buf);
+
+	return newbuf;
+}
+
+/*
+ * iso_to_utf8()
+ *
+ * konwertuje iso na utf8, zwalnia podany bufor i zwraca nowy.
+ */
+
+char *iso_to_utf8(char *b)
+{
+	unsigned char *buf = (unsigned char *) b;
+	char *newbuf;
+	int newlen = 0;
+	int i, j;
+
+	if (!buf)
+		return NULL;
+
+	for (i = 0; buf[i]; i++) {
+		unsigned short znak = (buf[i] < 0x80) ? buf[i] : table_iso_8859_2[buf[i]-0x80];
+
+		if (znak < 0x80)	newlen += 1;
+		else if (znak < 0x800)	newlen += 2;
+		else			newlen += 3;
+	}
+
+	newbuf = xmalloc(newlen+1);
+
+	for (i = 0, j = 0; buf[i]; i++) {
+		unsigned short znak = (buf[i] < 0x80) ? buf[i] : table_iso_8859_2[buf[i]-0x80];
+		int count;
+
+		if (znak < 0x80)	count = 1;
+		else if (znak < 0x800)	count = 2;
+		else			count = 3;
+
+		switch (count) {
+			case 3: newbuf[j+2] = 0x80 | (znak & 0x3f); znak = znak >> 6; znak |= 0x800;
+			case 2: newbuf[j+1] = 0x80 | (znak & 0x3f); znak = znak >> 6; znak |= 0xc0;
+			case 1: newbuf[j] = znak;
+		}
+		j += count;
+	}
+	newbuf[j] = '\0';
+
+	xfree(buf);
+
+	return newbuf;
+}
+
+
 /*
  * iso_to_cp()
  *
@@ -2052,7 +2325,15 @@ void iso_to_cp(unsigned char *buf)
 			0xBC, 0x9F,	/*  */
 			0xA1, 0xA5,	/*  */
 			0xA6, 0x8C,	/*  */
-			0xAC, 0x8F	/*  */
+			0xAC, 0x8F,	/*  */
+			0xB9, 0x9A,	/*  */
+			0xA9, 0x8A,	/*  */
+			0xBE, 0x9E,	/*  */
+			0xAE, 0x8E,	/*  */
+			0xBB, 0x9D,	/*  */
+			0xAB, 0x8D,	/*  */
+			0xB5, 0xBE,	/*  */
+			0xA5, 0xBC,	/*  */
 		};
 
 		charmap_prepare(charmap, conv, sizeof(conv), 1);
@@ -2543,6 +2824,9 @@ int ioctld_socket(const char *path)
 		usleep(usecs);
 	}
 
+	close(ioctld_sock);
+	ioctld_sock = -1;
+
         return -1;
 }
 
@@ -2607,7 +2891,7 @@ int init_control_pipe(const char *pipe_file)
 	if (!stat(pipe_file, &st) && !S_ISFIFO(st.st_mode))
 		err_str = saprintf("Plik %s nie jest potokiem. Ignoruj.\n", pipe_file);
 
-	if (mkfifo(pipe_file, 0600) < 0 && errno != EEXIST)
+	if (mkfifo(pipe_file, config_files_mode_config) < 0 && errno != EEXIST)
 		err_str = saprintf("Nie mog stworzy potoku %s: %s. Ignoruj.\n", pipe_file, strerror(errno));
 
 	if ((fd = open(pipe_file, O_RDWR | O_NONBLOCK)) < 0)
@@ -2642,6 +2926,29 @@ const char *timestamp(const char *format)
 	return buf;
 }
 
+/*
+ * timestamp_time()
+ *
+ * jak wyzej, ale na podstawie podanego czasu.
+ *
+ * - format
+ * - czas
+ */
+const char *timestamp_time(const char *format, time_t t)
+{
+	struct tm *tm;
+	static char buf[100];
+
+	if (!format || format[0] == '\0')
+		return itoa(t);
+
+	tm = localtime(&t);
+
+	if (!strftime(buf, sizeof(buf), format, tm))
+		return "TOOLONG";
+	return buf;
+}
+
 /*
  * unidle()
  *
@@ -2956,7 +3263,7 @@ void update_status_myip()
 			u->ip.s_addr = inet_addr(config_dcc_ip);
 		else {
 			struct sockaddr_in foo;
-			int bar = sizeof(foo);
+			socklen_t bar = sizeof(foo);
 
 			if (getsockname(sess->fd, (struct sockaddr *) &foo, &bar))
 				goto fail;
@@ -3025,6 +3332,28 @@ void change_status(int status, const char *xarg, int autom)
 			auto_format = "auto_back";
 			auto_format_descr = "auto_back_descr";
 			break;
+		case GG_STATUS_DND:
+		case GG_STATUS_DND_DESCR:
+			status_descr = GG_STATUS_DND_DESCR;
+			random_mask = 4;
+			filename = "dnd.reasons";
+			config_x_reason = config_dnd_reason;
+			format = "dnd";
+			format_descr = "dnd_descr";
+			auto_format = "auto_dnd";
+			auto_format_descr = "auto_dnd_descr";
+			break;
+		case GG_STATUS_FFC:
+		case GG_STATUS_FFC_DESCR:
+			status_descr = GG_STATUS_FFC_DESCR;
+			random_mask = 4;
+			filename = "ffc.reasons";
+			config_x_reason = config_ffc_reason;
+			format = "ffc";
+			format_descr = "ffc_descr";
+			auto_format = "auto_ffc";
+			auto_format_descr = "auto_ffc_descr";
+			break;
 		case GG_STATUS_INVISIBLE:
 		case GG_STATUS_INVISIBLE_DESCR:
 			status_descr = GG_STATUS_INVISIBLE_DESCR;
@@ -3137,9 +3466,9 @@ void change_status(int status, const char *xarg, int autom)
 
 	if (sess && sess->state == GG_STATE_CONNECTED) {
 		if (config_reason) {
-		    	iso_to_cp(config_reason);
+		    	iso_to_cp((unsigned char *) config_reason);
 			gg_change_status_descr(sess, config_status, config_reason);
-			cp_to_iso(config_reason);
+			cp_to_iso((unsigned char *) config_reason);
 		} else
 		    	gg_change_status(sess, config_status);
 	}
@@ -3184,6 +3513,18 @@ const char *ekg_status_label(int status, const char *prefix)
 		case GG_STATUS_BUSY_DESCR:
 			label = "busy_descr";
 			break;
+		case GG_STATUS_FFC:
+			label = "ffc";
+			break;
+		case GG_STATUS_FFC_DESCR:
+			label = "ffc_descr";
+			break;
+		case GG_STATUS_DND:
+			label = "dnd";
+			break;
+		case GG_STATUS_DND_DESCR:
+			label = "dnd_descr";
+			break;
 		case GG_STATUS_INVISIBLE:
 			label = "invisible";
 			break;
@@ -3446,7 +3787,7 @@ time_t parsetimestr(const char *p)
  *  - path: backup nie by potrzebny lub config_dcc_backups == 0
  *  - pozostae wartoci: wskanik do nowej cieki
  */
-unsigned char *unique_name (unsigned char *path)
+unsigned char *unique_name(unsigned char *path)
 {
 	struct stat st;
 	int num;
@@ -3467,3 +3808,41 @@ unsigned char *unique_name (unsigned char *path)
 	return (num == 1000) ? NULL : newpath;
 }
 
+/*
+ * get_line()
+ *
+ * pobiera lini tekstu z bufora. funkcja niszczy bufor rdowy 
+ * bezpowrotnie, dzielc go na kolejne cigi znakw i obcina znaki 
+ * koca linii.
+ *
+ * - ptr - wskanik do zmiennej, ktra przechowuje aktualne pooenie
+ *   w analizowanym buforze.
+ *
+ * wskanik do kolejnej linii tekstu lub NULL, jeeli to ju koniec
+ * bufora.
+ */
+
+char *get_line(char **ptr)
+{
+	char *foo, *res;
+
+	if (!ptr || !*ptr || !strcmp(*ptr, ""))
+		return NULL;
+
+	res = *ptr;
+
+	if (!(foo = strchr(*ptr, '\n')))
+		*ptr += strlen(*ptr);
+	else {
+		size_t len;
+		*ptr = foo + 1;
+		*foo = 0;
+
+		len = strlen(res);
+
+		if (len > 1 && res[len - 1] == '\r')
+			res[len - 1] = 0;
+	}
+
+	return res;
+}
