@@ -1,4 +1,4 @@
-/* $Id: stuff.h,v 1.277.2.4 2007-04-28 20:02:25 porridge Exp $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -70,9 +70,11 @@ enum event_t {
 	EVENT_DISCONNECTED = TOGGLE_BIT(23),
 	EVENT_CONNECTIONLOST = TOGGLE_BIT(24),
 	EVENT_IMAGE = TOGGLE_BIT(25),
+	EVENT_DND = TOGGLE_BIT(26),
+	EVENT_FFC = TOGGLE_BIT(27),
 
-	EVENT_ALL = TOGGLE_BIT(26) - 1,
-	INACTIVE_EVENT = TOGGLE_BIT(26)
+	EVENT_ALL = TOGGLE_BIT(28) - 1,
+	INACTIVE_EVENT = TOGGLE_BIT(28)
 };
 
 struct event_label {
@@ -100,6 +102,7 @@ struct transfer {
 	uin_t uin;
 	char *filename;
 	struct gg_dcc *dcc;
+	struct gg_dcc7 *dcc7;
 	time_t start;
 	int type;
 	int id;
@@ -233,6 +236,8 @@ char *config_aspell_lang;
 char *config_aspell_encoding;
 #endif
 char *config_back_reason;
+char *config_ffc_reason;
+char *config_dnd_reason;
 int config_beep;
 int config_beep_msg;
 int config_beep_chat;
@@ -272,6 +277,10 @@ int config_emoticons;
 int config_encryption;
 int config_enter_scrolls;
 int config_events_delay;
+int config_files_mode_config;
+int config_files_mode_received;
+int config_files_mode_config_int;
+int config_files_mode_received_int;
 char *config_interface;
 int config_header_size;
 int config_ignore_unknown_sender;
@@ -291,6 +300,9 @@ char *config_log_timestamp;
 int config_make_window;
 int config_msg_as_chat;
 int config_mesg;
+#ifdef WITH_UI_NCURSES
+int config_mouse;
+#endif
 char *config_nick;
 char *config_password;
 int config_password_cp1250;
@@ -351,6 +363,7 @@ char *home_dir;
 char *config_dir;
 int command_processing;
 int in_autoexec;
+int no_autorun;
 int reconnect_timer;
 time_t last_action;
 int connecting;
@@ -410,6 +423,7 @@ void changed_proxy(const char *var);
 void changed_theme(const char *var);
 void changed_uin(const char *var);
 void changed_xxx_reason(const char *var);
+void changed_files_mode(const char *var);
 
 struct conference *conference_add(const char *string, const char *nicklist, int quiet);
 int conference_remove(const char *name, int quiet);
@@ -438,8 +452,10 @@ void event_free(void);
 int mesg_set(int what);
 int msg_encrypt(uin_t uin, unsigned char **msg);
 void cp_to_iso(unsigned char *buf);
+char *utf8_to_iso(char *buf);
 void iso_to_cp(unsigned char *buf);
 void iso_to_ascii(unsigned char *buf);
+char *iso_to_utf8(char *buf);
 char *strip_chars(char *line, unsigned char what); 
 char *strip_spaces(char *line);
 
@@ -462,6 +478,7 @@ int ioctld_send(const char *seq, int act, int quiet);
 int init_control_pipe(const char *path);
 
 const char *timestamp(const char *format);
+const char *timestamp_time(const char *format, time_t t);
 void unidle(void);
 int on_off(const char *value);
 int transfer_id(void);
@@ -491,7 +508,9 @@ void update_status_myip(void);
 void change_status(int status, const char *arg, int autom);
 const char *ekg_status_label(int status, const char *prefix);
 int ekg_hide_descr_status(int status);
-unsigned char *unique_name (unsigned char *path);
+unsigned char *unique_name(unsigned char *path);
+
+char *get_line(char **ptr);
 
 /* funkcje poza stuff.c */
 void ekg_wait_for_key(void);
