@@ -1,7 +1,7 @@
-/* $Id: themes.c,v 1.342.2.5 2007-04-28 20:02:25 porridge Exp $ */
+/* $Id$ */
 
 /*
- *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
+ *  (C) Copyright 2001-2007 Wojtek Kaniewski <wojtekka@irc.pl>
  * 			    Piotr Kupisiewicz <deli@rzepaknet.us>
  *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
@@ -369,7 +369,13 @@ char *va_format_string(const char *format, va_list ap)
 		theme_cache_reset();
 
 	if (!config_display_pl_chars)
-		iso_to_ascii(buf->str);
+		iso_to_ascii((unsigned char *) buf->str);
+
+#ifdef WITH_UI_GTK
+	if (ui_init == ui_gtk_init) {
+		buf->str = iso_to_utf8(buf->str);
+	}
+#endif
 
 	return string_free(buf, 0);
 }
@@ -872,7 +878,11 @@ void theme_init()
 	format_add("generic", "%> %1\n", 1);
 	format_add("generic2", "%) %1\n", 1);
 	format_add("generic_error", "%! %1\n", 1);
-	format_add("debug", "%n%1\n", 1);
+	format_add("debug", 	"%n%1\n", 1);
+	format_add("fdebug",	"%b%1\n", 1);
+	format_add("iodebug",	"%y%1\n", 1);
+	format_add("iorecvdebug", "%Y%1\n", 1);
+	format_add("edebug",	"%R%1\n", 1);
 	format_add("not_enough_params", "%! Za mao parametrw. Sprbuj %Thelp %1%n\n", 1);
 	format_add("invalid_params", "%! Nieprawidowe parametry. Sprbuj %Thelp %1%n\n", 1);
 	format_add("invalid_uin", "%! Nieprawidowy numer uytkownika\n", 1);
@@ -904,6 +914,10 @@ void theme_init()
 	format_add("away_descr", "%> Zmieniono stan na zajty: %T%1%n%2\n", 1);
 	format_add("back", "%> Zmieniono stan na dostpny\n", 1);
 	format_add("back_descr", "%> Zmieniono stan na dostpny: %T%1%n%2%n\n", 1);
+	format_add("dnd", "%> Zmieniono stan na nie przeszkadza\n", 1);
+	format_add("dnd_descr", "%> Zmieniono stan na nie przeszkadza: %T%1%n%2%n\n", 1);
+	format_add("ffc", "%> Zmieniono stan na poGGadaj ze mn\n", 1);
+	format_add("ffc_descr", "%> Zmieniono stan na poGGadaj ze mn: %T%1%n%2%n\n", 1);
 	format_add("invisible", "%> Zmieniono stan na niewidoczny\n", 1);
 	format_add("invisible_descr", "%> Zmieniono stan na niewidoczny: %T%1%n%2\n", 1);
 	format_add("private_mode_is_on", "%> Tryb ,,tylko dla znajomych'' jest wczony\n", 1);
@@ -949,6 +963,10 @@ void theme_init()
 	format_add("list_avail_descr", "%> %1 %Y(dostpn%@2: %n%5%Y)%n %b%3:%4%n\n", 1);
 	format_add("list_busy", "%> %1 %G(zajt%@2)%n %b%3:%4%n\n", 1);
 	format_add("list_busy_descr", "%> %1 %G(zajt%@2: %n%5%G)%n %b%3:%4%n\n", 1);
+	format_add("list_ffc", "%> %1 %y(poGGadaj ze mn)%n %b%3:%4%n\n", 1);
+	format_add("list_ffc_descr", "%> %1 %y(poGGadaj ze mn: %n%5%y)%n %b%3:%4%n\n", 1);
+	format_add("list_dnd", "%> %1 %g(nie przeszkadza)%n %b%3:%4%n\n", 1);
+	format_add("list_dnd_descr", "%> %1 %g(nie przeszkadza: %n%5%g)%n %b%3:%4%n\n", 1);
 	format_add("list_not_avail", "%> %1 %r(niedostpn%@2)%n\n", 1);
 	format_add("list_not_avail_descr", "%> %1 %r(niedostpn%@2: %n%5%r)%n\n", 1);
 	format_add("list_invisible", "%> %1 %c(niewidoczn%@2)%n %b%3:%4%n\n", 1);
@@ -974,6 +992,16 @@ void theme_init()
 	format_add("contacts_busy_descr", "%Ki%G%1%n", 1);
 	format_add("contacts_busy_descr_full", "%Ki%G%1%n %2", 1);
 	format_add("contacts_busy_footer", "", 1);
+	format_add("contacts_ffc_header", "", 1);
+	format_add("contacts_ffc", " %y%1%n", 1);
+	format_add("contacts_ffc_descr", "%Ki%y%1%n", 1);
+	format_add("contacts_ffc_descr_full", "%Ki%y%1%n %2", 1);
+	format_add("contacts_ffc_footer", "", 1);
+	format_add("contacts_dnd_header", "", 1);
+	format_add("contacts_dnd", " %g%1%n", 1);
+	format_add("contacts_dnd_descr", "%Ki%g%1%n", 1);
+	format_add("contacts_dnd_descr_full", "%Ki%g%1%n %2", 1);
+	format_add("contacts_dnd_footer", "", 1);
 	format_add("contacts_not_avail_header", "", 1);
 	format_add("contacts_not_avail", " %r%1%n", 1);
 	format_add("contacts_not_avail_descr", "%Ki%r%1%n", 1);
@@ -1056,6 +1084,10 @@ void theme_init()
 	format_add("status_avail_descr", "%> %1 jest dostpn%@2: %T%3%n\n", 1);
 	format_add("status_busy", "%> %1 jest zajt%@2\n", 1);
 	format_add("status_busy_descr", "%> %1 jest zajt%@2: %T%3%n\n", 1);
+	format_add("status_dnd", "%> %1 jest nie przeszkadza\n", 1);
+	format_add("status_dnd_descr", "%> %1 jest nie przeszkadza: %T%3%n\n", 1);
+	format_add("status_ffc", "%> %1 chce poGGada\n", 1);
+	format_add("status_ffc_descr", "%> %1 chce poGGada: %T%3%n\n", 1);
 	format_add("status_not_avail", "%> %1 jest niedostpn%@2\n", 1);
 	format_add("status_not_avail_descr", "%> %1 jest niedostpn%@2: %T%3%n\n", 1);
 	format_add("status_invisible", "%> %1 jest niewidoczn%@2\n", 1);
@@ -1066,6 +1098,10 @@ void theme_init()
 	format_add("auto_away_descr", "%> Automagicznie zmieniono stan na zajty po %1 nieaktywnoci: %T%2%n%3\n", 1);
 	format_add("auto_back", "%> Automagicznie zmieniono stan na dostpny\n", 1);
 	format_add("auto_back_descr", "%> Automagicznie zmieniono stan na dostpny: %T%2%n%3\n", 1);
+	format_add("auto_dnd", "%> Automagicznie zmieniono stan na nie przeszkadza\n", 1);
+	format_add("auto_dnd_descr", "%> Automagicznie zmieniono stan na nie przeszkadza: %T%2%n%3\n", 1);
+	format_add("auto_ffc", "%> Automagicznie zmieniono stan na poGGadaj ze mn\n", 1);
+	format_add("auto_ffc_descr", "%> Automagicznie zmieniono stan na poGGadaj ze mn: %T%2%n%3\n", 1);
 
 	/* poczenie z serwerem */
 	format_add("connecting", "%> cz si z serwerem...\n", 1);
@@ -1173,7 +1209,7 @@ void theme_init()
 	format_add("search_results_multi_unknown", "-", 1);
 /*	format_add("search_results_multi_female", "k", 1); */
 /*	format_add("search_results_multi_male", "m", 1); */
-	format_add("search_results_multi", "%7 %[-7]1 %K|%n %[12]3 %K|%n %[12]2 %K|%n %[4]5 %K|%n %[12]4\n", 1);
+	format_add("search_results_multi", "%7 %[-8]1 %K|%n %[12]3 %K|%n %[12]2 %K|%n %[4]5 %K|%n %[12]4\n", 1);
 
 	format_add("search_results_single_active", "%Y(dostpn%@1)%n", 1);
 	format_add("search_results_single_busy", "%G(zajt%@1)%n", 1);
@@ -1218,6 +1254,10 @@ void theme_init()
 	format_add("user_info_avail_descr", "%Ydostpn%@1%n (%2)", 1);
 	format_add("user_info_busy", "%Gzajt%@1%n", 1);
 	format_add("user_info_busy_descr", "%Gzajt%@1%n (%2)", 1);
+	format_add("user_info_ffc", "%ypoGGadaj ze mn%n", 1);
+	format_add("user_info_ffc_descr", "%ypoGGadaj ze mn%n (%2)", 1);
+	format_add("user_info_dnd", "%gnie przeszkadza%n", 1);
+	format_add("user_info_dnd_descr", "%gnie przeszkadza%n (%2)", 1);
 	format_add("user_info_not_avail", "%rniedostpn%@1%n", 1);
 	format_add("user_info_not_avail_descr", "%rniedostpn%@1%n (%2)", 1);
 	format_add("user_info_invisible", "%cniewidoczn%@1%n", 1);
@@ -1242,6 +1282,10 @@ void theme_init()
 	format_add("show_status_avail_descr", "%Ydostpny%n (%T%1%n%2)", 1);
 	format_add("show_status_busy", "%Gzajty%n", 1);
 	format_add("show_status_busy_descr", "%Gzajty%n (%T%1%n%2)", 1);
+	format_add("show_status_dnd", "%gnie przeszkadza%n", 1);
+	format_add("show_status_dnd_descr", "%gnie przeszkadza%n (%T%1%n%2)", 1);
+	format_add("show_status_ffc", "%ypoGGadaj ze mn%n", 1);
+	format_add("show_status_ffc_descr", "%ypoGGadaj ze mn%n (%T%1%n%2)", 1);
 	format_add("show_status_invisible", "%cniewidoczny%n", 1);
 	format_add("show_status_invisible_descr", "%cniewidoczny%n (%T%1%n%2)", 1);
 	format_add("show_status_not_avail", "%rniedostpny%n", 1);
@@ -1274,6 +1318,7 @@ void theme_init()
 	format_add("dcc_spoof", "%! %|Uytkownik %1 poda serwerowi adres %T%2%n, ale poczy si z nami z adresu %T%3%n. Moliwe, e kto prbuje si pod niego podszy. Zachowaj ostrono!\n", 1);
 	format_add("dcc_limit", "%! %|Przekroczono limit bezporednich pocze i dla bezpieczestwa zostay one wyczone. Aby je wczy ponownie, naley wpisa polecenie %Tset dcc 1%n i poczy si ponownie. Limit mona zmieni za pomoc zmiennej %Tdcc_limit%n.\n", 1);
 	format_add("dcc_create_error", "%! Nie mona wczy pocze bezporednich: %1\n", 1);
+	format_add("dcc_error", "%! Bd transmisji: %1\n", 1);
 	format_add("dcc_error_network", "%! Bd transmisji z %1\n", 1);
 	format_add("dcc_error_refused", "%! Poczenie z %1 zostao odrzucone\n", 1);
 	format_add("dcc_error_unknown", "%! Nieznany bd poczenia bezporedniego\n", 1);
@@ -1358,6 +1403,10 @@ void theme_init()
 	format_add("quick_list_avail,speech", "%1 jest dostpny, ", 1);
 	format_add("quick_list_busy", " %G%1%n", 1);
 	format_add("quick_list_busy,speech", "%1 jest zajty, ", 1);
+	format_add("quick_list_ffc", " %y%1%n", 1);
+	format_add("quick_list_ffc,speech", "%1 jest poggadaj ze mn, ", 1);
+	format_add("quick_list_dnd", " %g%1%n", 1);
+	format_add("quick_list_dnd,speech", "%1 jest nie przeszkadza, ", 1);
 	format_add("quick_list_invisible", " %c%1%n", 1);
 
 	/* window */
@@ -1374,6 +1423,7 @@ void theme_init()
 	format_add("window_dump_error", "%! Bd zapisu do pliku %T%1%n\n", 1);
 	format_add("window_dump_done", "%! Zapisano\n", 1);
 	format_add("window_day_changed", "%) Zmiana daty: %T%1%n -> %T%2%n\n", 1);
+	format_add("window_swapped", "%) Okna %T%1%n i %T%2%n zamienione miejscami\n", 1);
 
 	/* bind */
 	format_add("bind_seq_incorrect", "%! Sekwencja %T%1%n jest nieprawidowa\n", 1); 
