@@ -1,4 +1,4 @@
-/* $Id: ui.h,v 1.31 2006-06-10 20:56:28 gophi Exp $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2002-2004 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -69,4 +69,9 @@ extern void header_statusbar_resize(const char *name);
 
 #endif
 
+#ifdef WITH_UI_GTK
+extern void ui_gtk_init(void);
+/* rest in ui-gtk.h */
+#endif
+
 #endif /* __UI_H */
