@@ -1,4 +1,4 @@
-/* $Id: userlist.c,v 1.93.2.4 2007-04-28 20:02:26 porridge Exp $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2001-2006 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -66,6 +66,7 @@ struct ignore_label ignore_labels[IGNORE_LABELS_COUNT + 1] = {
 	{ IGNORE_DCC, "dcc" },
 	{ IGNORE_EVENTS, "events" },
 	{ IGNORE_SMSAWAY, "smsaway" },
+	{ IGNORE_DISPLAY, "display" },
 	{ 0, NULL }
 };
 
@@ -211,11 +212,12 @@ int userlist_set(const char *contacts, int config)
 	
 	contsave = cont = xstrdup(contacts);
 	
-	while ((buf = gg_get_line(&cont))) {
+	while ((buf = get_line(&cont))) {
 		struct userlist u;
 		char **entry, *uin;
 		int i, count;
-		
+		char *tmp;
+
 		memset(&u, 0, sizeof(u));
 			
 		if (buf[0] == '#' || (buf[0] == '/' && buf[1] == '/'))
@@ -260,22 +262,30 @@ int userlist_set(const char *contacts, int config)
 			}
 		}
 
-		u.first_name = xstrdup(entry[0]);
-		u.last_name = xstrdup(entry[1]);
-		u.nickname = xstrdup(entry[2]);
-		if (entry[3] && !valid_nick(entry[3]))
-			u.display = saprintf("_%s", entry[3]);
-		else
-			u.display = xstrdup(entry[3]);
-		u.mobile = xstrdup(entry[4]);
-		u.groups = group_init(entry[5]);
+		u.first_name = unescape(entry[0]);
+		u.last_name = unescape(entry[1]);
+		u.nickname = unescape(entry[2]);
+		u.display = unescape(entry[3]);
+		if (u.display && !valid_nick(u.display)) {
+			tmp = u.display;
+			u.display = saprintf("_%s", tmp);
+			free(tmp);
+		}
+		u.mobile = unescape(entry[4]);
+		tmp = unescape(entry[5]);
+		u.groups = group_init(tmp);
+		free(tmp);
 		u.status = GG_STATUS_NOT_AVAIL;
 
 		/* mamy adres e-mail? */
 		if (count > 7) {
-			u.email = xstrdup(entry[7]);
+			u.email = unescape(entry[7]);
 			if (count > 8)
-				u.foreign = saprintf(";%s", entry[8]);
+			{
+				tmp = unescape(entry[8]);
+				u.foreign = saprintf(";%s", tmp);
+				free(tmp);
+			}
 		}
 
 		array_free(entry);
@@ -297,9 +307,9 @@ int userlist_set(const char *contacts, int config)
 
 		if (sess && sess->state == GG_STATE_CONNECTED) {
 			if (config_reason) {
-				iso_to_cp(config_reason);
+				iso_to_cp((unsigned char *) config_reason);
 				gg_change_status_descr(sess, config_status, config_reason);
-				cp_to_iso(config_reason);
+				cp_to_iso((unsigned char *) config_reason);
 			} else
 				gg_change_status(sess, config_status);
 		}
@@ -403,7 +413,7 @@ int userlist_write(int pid)
 		return -2;
 	}
 
-	fchmod(fileno(f), 0600);
+	fchmod(fileno(f), config_files_mode_config);
 	
 	fputs(contacts, f);
 	xfree(contacts);
@@ -437,7 +447,7 @@ int userlist_write_wap()
 	if (!(f = fopen(filename, "w")))
 		return -1;
 
-	fchmod(fileno(f), 0600);
+	fchmod(fileno(f), config_files_mode_config);
 	fprintf(f, "%s\n", (sess) ? "C" : "D");
 
 	for (l = userlist; l; l = l->next) {
