@@ -1,4 +1,4 @@
-/* $Id: vars.c,v 1.184.2.4 2007-04-28 20:02:26 porridge Exp $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -127,6 +127,8 @@ void variable_init()
 #endif
 	variable_add("away_reason", "ar", VAR_STR, 1, &config_away_reason, changed_xxx_reason, NULL, NULL);
 	variable_add("back_reason", "br", VAR_STR, 1, &config_back_reason, changed_xxx_reason, NULL, NULL);
+	variable_add("ffc_reason", "fr", VAR_STR, 1, &config_ffc_reason, changed_xxx_reason, NULL, NULL);
+	variable_add("dnd_reason", "dR", VAR_STR, 1, &config_dnd_reason, changed_xxx_reason, NULL, NULL);
 #ifdef WITH_UI_NCURSES
 	if (ui_init == ui_ncurses_init) {
 		variable_add("backlog_overlap", "bo", VAR_INT, 1, &config_backlog_overlap, NULL, NULL, NULL);
@@ -164,8 +166,20 @@ void variable_init()
 #else
 	variable_add("ctrld_quits", "cq", VAR_BOOL, 2, NULL, NULL, NULL, NULL);
 #endif
+#if defined WITH_UI_NCURSES || defined WITH_UI_GTK
+	if (
 #ifdef WITH_UI_NCURSES
-	if (ui_init == ui_ncurses_init) {
+		ui_init == ui_ncurses_init
+#endif
+
+#if defined WITH_UI_NCURSES && defined WITH_UI_GTK
+						||
+#endif
+
+#ifdef WITH_UI_GTK
+							ui_init == ui_gtk_init
+#endif
+		) {
 		variable_add("contacts", "co", VAR_INT, 1, &config_contacts, NULL, NULL, NULL);
 		variable_add("contacts_groups", "cg", VAR_STR, 1, &config_contacts_groups, NULL, NULL, dd_contacts);
 		variable_add("contacts_options", "cO", VAR_STR, 1, &config_contacts_options, NULL, NULL, dd_contacts);
@@ -236,6 +250,8 @@ void variable_init()
 	variable_add("enter_scrolls", "es", VAR_BOOL, 2, NULL,  NULL, NULL, NULL);
 #endif
 	variable_add("events_delay", "ev", VAR_INT, 1, &config_events_delay, NULL, NULL, NULL);
+	variable_add("files_mode_config", "Mc", VAR_INT, 1, &config_files_mode_config_int, changed_files_mode, NULL, NULL);
+	variable_add("files_mode_received", "Mr", VAR_INT, 1, &config_files_mode_received_int, changed_files_mode, NULL, NULL);
 #ifdef WITH_UI_NCURSES
 	if (ui_init == ui_ncurses_init)
 		variable_add("header_size", "hs", VAR_INT, 1, &config_header_size, header_statusbar_resize, NULL, NULL);
@@ -259,6 +275,11 @@ void variable_init()
 	variable_add("make_window", "mw", VAR_MAP, 1, &config_make_window, NULL, variable_map(4, 0, 0, "none", 1, 2, "usefree", 2, 1, "always", 4, 0, "nomsg"), NULL);
 	variable_add("msg_as_chat", "mc", VAR_INT, 1, &config_msg_as_chat, NULL, variable_map(3, 0, 0, "no", 1, 2, "sendonly", 2, 1, "yes"), NULL);
 	variable_add("mesg", "ma", VAR_INT, 1, &config_mesg, changed_mesg, variable_map(3, 0, 0, "no", 1, 2, "yes", 2, 1, "default"), NULL);
+#ifdef WITH_UI_NCURSES
+	variable_add("mouse", "mo", VAR_BOOL, 1, &config_mouse, NULL, NULL, NULL);
+#else
+	variable_add("mouse", "mo", VAR_BOOL, 2, NULL, NULL, NULL, NULL);
+#endif
 	variable_add("nick", "ni", VAR_STR, 1, &config_nick, NULL, NULL, NULL);
 	variable_add("proxy", "pr", VAR_STR, 1, &config_proxy, changed_proxy, NULL, NULL);
 	variable_add("proxy_forwarding", "pf", VAR_STR, 1, &config_proxy_forwarding, NULL, NULL, NULL);
